Agda-2.6.3.1: A dependently typed functional programming language and proof assistant
Safe HaskellNone
LanguageHaskell2010

Agda.Utils.Lens

Description

A cut-down implementation of lenses, with names taken from Edward Kmett's lens package.

Synopsis

Documentation

key :: Ord k => k -> Lens' (Maybe v) (Map k v) Source #

(.=) :: MonadState o m => Lens' i o -> i -> m () infix 4 Source #

Write a part of the state.

over :: Lens' i o -> LensMap i o Source #

Modify inner part i of structure o using a function i -> i.

type Lens' i o = forall (f :: Type -> Type). Functor f => (i -> f i) -> o -> f o Source #

Van Laarhoven style homogeneous lenses. Mnemoic: "Lens inner outer".

set :: Lens' i o -> LensSet i o Source #

Set inner part i of structure o as designated by Lens' i o.

type LensGet i o = o -> i Source #

type LensSet i o = i -> o -> o Source #

type LensMap i o = (i -> i) -> o -> o Source #

lFst :: forall a b f. Functor f => (a -> f a) -> (a, b) -> f (a, b) Source #

lSnd :: forall b a f. Functor f => (b -> f b) -> (a, b) -> f (a, b) Source #

(^.) :: o -> Lens' i o -> i infixl 8 Source #

Get inner part i of structure o as designated by Lens' i o.

focus :: forall (m :: Type -> Type) i o a. Monad m => Lens' i o -> StateT i m a -> StateT o m a Source #

Focus on a part of the state for a stateful computation.

use :: MonadState o m => Lens' i o -> m i Source #

Read a part of the state.

(%=) :: MonadState o m => Lens' i o -> (i -> i) -> m () infix 4 Source #

Modify a part of the state.

(%==) :: MonadState o m => Lens' i o -> (i -> m i) -> m () infix 4 Source #

Modify a part of the state monadically.

(%%=) :: MonadState o m => Lens' i o -> (i -> m (i, r)) -> m r infix 4 Source #

Modify a part of the state monadically, and return some result.

locallyState :: MonadState o m => Lens' i o -> (i -> i) -> m r -> m r Source #

Modify a part of the state locally.

view :: MonadReader o m => Lens' i o -> m i Source #

Ask for part of read-only state.

locally :: MonadReader o m => Lens' i o -> (i -> i) -> m a -> m a Source #

Modify a part of the state in a subcomputation.

locally' :: ((o -> o) -> m a -> m a) -> Lens' i o -> (i -> i) -> m a -> m a Source #

(<&>) :: Functor m => m a -> (a -> b) -> m b infixl 1 Source #

Infix version of for.